package de.intevation.flys.artifacts.model;

import de.intevation.artifactdatabase.state.Facet;
import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;
import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.math.Linear;
import de.intevation.flys.artifacts.states.DefaultState.ComputeType;
import de.intevation.flys.artifacts.model.extreme.ExtremeResult;

import org.apache.log4j.Logger;

/**
 * Facet of a Waterlevel (WQKms).
 */
public class WaterlevelFacet extends DataFacet {

    private static Logger logger = Logger.getLogger(WaterlevelFacet.class);

    public WaterlevelFacet(int index, String name, String description) {
        super(index, name, description, ComputeType.ADVANCE, null, null);
    }

    public WaterlevelFacet(
        int         index,
        String      name,
        String      description,
        ComputeType type,
        String      stateID,
        String      hash
    ) {
        super(index, name, description, type, hash, stateID);
    }

    public WaterlevelFacet() {
    }

    protected WQKms [] getWQKms(CalculationResult res) {
        if (res.getData() instanceof ExtremeResult)
            return ((ExtremeResult) res.getData()).getWQKms();
        else if (res.getData() instanceof WQKms[]) {
            return (WQKms []) res.getData();
        }
        else {
            logger.error("WaterlevelFacet got wrong data type " + res.getData());
            return null;
        }
    }

    /**
     * Get waterlevel data.
     * @return a WQKms at given index.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {

        if (logger.isDebugEnabled()) {
            logger.debug("Get data for waterlevels at index: " + index +
                " /stateId: " + stateId);
        }

        if (artifact == null) {
            logger.error("WaterlevelFacet.getData: artifact is null");
            return null;
        }

        FLYSArtifact winfo = (FLYSArtifact) artifact;

        CalculationResult res = (CalculationResult)
            winfo.compute(context, hash, stateId, type, false);

        if (res == null) {
            logger.error("WaterlevelFacet.getData: null result");
            return null;
        }

        WQKms [] wqkms = getWQKms(res);
        Object KM = context.getContextValue("currentKm");
        if (KM != null) {
            logger.debug("interpolate at given km");
            // TODO handle exact match.

            WQKms wqkmsI = wqkms[index];
            double km = (Double)KM;

            // TODO employ DataUtils interface to TDoubleArraList
            int size = wqkmsI.size();
            boolean kmIncreasing = wqkmsI.getKm(0) < wqkmsI.getKm(size-1);
            int mod = kmIncreasing ? +1 : -1;
            int idx = 0;
            if (!kmIncreasing) {
                while (idx < size && wqkmsI.getKm(idx) < km) {
                    idx++;
                }
            }
            else {
                idx = size-1;
                while (idx > 0 && wqkmsI.getKm(idx) > km) {
                    idx--;
                }
            }

            WQKms resultWQKms = new WQKms();
            if ((idx != -1) && (idx < size) && (idx - mod != -1) && (idx - mod < size)) {
                double inW = Linear.linear(
                    km,
                    wqkmsI.getKm(idx), wqkmsI.getKm(idx - mod),
                    wqkmsI.getW(idx), wqkmsI.getW(idx - mod));
                double inQ = Linear.linear(
                    km,
                    wqkmsI.getKm(idx), wqkmsI.getKm(idx - mod),
                    wqkmsI.getQ(idx), wqkmsI.getQ(idx - mod));
                resultWQKms.add(inW, inQ, km);
            }

            return resultWQKms;
        }

        return wqkms != null ? wqkms[index] : null;
    }


    /** Copy deeply. */
    @Override
    public Facet deepCopy() {
        WaterlevelFacet copy = new WaterlevelFacet();
        copy.set(this);
        copy.type    = type;
        copy.hash    = hash;
        copy.stateId = stateId;
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
